@@ -201,8 +201,7 @@ window_is_fullscreen_size (MetaWindow *window)
 {
   int i;
 
-  if (window->rect.width >= window->screen->width &&
-      window->rect.height >= window->screen->height)
+  if (meta_rectangle_could_fit_rect (&window->rect, &window->screen->rect))
     {
       /* we use the work area since windows that try to
        * position at 0,0 will get pushed down by menu panel
@@ -210,26 +209,20 @@ window_is_fullscreen_size (MetaWindow *window)
       MetaRectangle workarea;
       
       meta_window_get_work_area_current_xinerama (window, &workarea);
-      if (window->rect.x <= workarea.x &&
-          window->rect.y <= workarea.y &&
-	  window->rect.x + window->rect.width >= workarea.x + workarea.width &&
-	  window->rect.y + window->rect.height >= workarea.y + workarea.height) 
+      if (meta_rectangle_contains_rect (&window->rect, &workarea))
         return TRUE;
     }
   
   i = 0;
   while (i < window->screen->n_xinerama_infos)
     {
-      if (window->rect.width >= window->screen->xinerama_infos[i].width &&
-          window->rect.height >= window->screen->xinerama_infos[i].height)
+      if (meta_rectangle_could_fit_rect (&window->rect,
+                                         &window->screen->xinerama_infos[i].rect))
         {
           MetaRectangle workarea;
           
           meta_window_get_work_area_current_xinerama (window, &workarea);
-          if (window->rect.x <= workarea.x &&
-              window->rect.y <= workarea.y &&
-	      window->rect.x + window->rect.width >= workarea.x + workarea.width &&
-	      window->rect.y + window->rect.height >= workarea.y + workarea.height) 
+          if (meta_rectangle_contains_rect (&window->rect, &workarea))
             return TRUE;
         }
       
